within ;
package Frvs
  model glucoseEquation
    parameter Real QL( unit = "mg.h-1") =   8400 "Glucose Input Flow Rate";
    parameter Real lambda =   2470 "constant";
    parameter Real v( unit="mU-1.h-1")= 139000
      "spotreba inzulin-dependentnich tkani. Pokles na 20 % simuluje diabetes melitus 2. typu - inzulin independentni";
    parameter Real mi =   7200 "constant";
    parameter Real theta =   2.5 "Reabsorption Maximum";
    parameter Real phi =   0.51 "Threshold Glucose Concentration";
    parameter Real alpha(unit="ml.h-1") =   7600 "Insulin Destruction Gain";
    parameter Real beta( unit = "mIU.ml.mg-1.h-1") =   1430
      "Insulin production rate - pokles na 20 % simuluje diabetes melitus 1. typu - inzulin dependentni";
    Real TURind "Tissue utilization rate - insulin independent";
    Real TURdep "Tissue utilization rate - insulin dependent";
    Real RLR( unit="mg.h-1") "Renal loss rate";
    constant Real nur( unit="l.h-1")= 0.06 "normalUrinationRate";
    Real guc(unit="mg.l-1") "Glucose concentration in urine";
    Real x( start=0.81) "Blood Glucose Concentration (mg/mL)";
    Real y( start=0.055) "Blood Insulin Concentration (mU/ML)";
    parameter Real CG = 15000 "Capacitance of Glucose";
    parameter Real CI = 15000 "Capacitance of Insulin";
    parameter Real GlucoseInfusion( unit="g.h-1");

    Modelica.Blocks.Interfaces.RealInput InsulineShot
      annotation (Placement(transformation(extent={{-120,10},{-80,50}})));
      Modelica.Blocks.Interfaces.RealInput GlucoseShot( unit = "g.h-1")
      annotation (Placement(transformation(extent={{-118,-70},{-80,-32}})));

    Modelica.Blocks.Interfaces.RealOutput xOutput
      annotation (Placement(transformation(extent={{80,80},{100,100}})));
    Modelica.Blocks.Interfaces.RealOutput yOutput
      annotation (Placement(transformation(extent={{80,-96},{100,-76}})));
  initial equation
    x = 0.81;
    y = 0.055;
  equation
    xOutput = x;
    yOutput = y;

    guc = RLR/nur;
    TURind = lambda * x;
    TURdep = v * x * y;
    if x <= theta then
      RLR = 0;
    else
      RLR = mi * (x - theta);
    end if;
    CG * der(x) = (GlucoseInfusion + GlucoseShot)*1000 + QL - TURind - TURdep - RLR;
    if x <= phi then
      CI * der(y) = -alpha * y + InsulineShot;
    else
      CI * der(y) = -alpha * y + InsulineShot + beta * (x - phi);
    end if;
    annotation(experiment(StartTime = 0.0, StopTime = 10, Tolerance = 1e-006), Diagram(
          graphics),
               uses(Modelica(version = "2.2.2")));
  end glucoseEquation;

  model regulaceGlukozy
    Modelica.Blocks.Sources.Pulse pulse(
      width=0.25,
      period=10,
      offset=0,
      startTime=0.5,
      amplitude=1e6)
      annotation (Placement(transformation(extent={{-60,-38},{-40,-18}})));
    Krev krev annotation (Placement(transformation(extent={{20,-14},{40,6}})));
    Slinivka slinivka
      annotation (Placement(transformation(extent={{-64,8},{-44,28}})));
  equation
    connect(slinivka.y, krev.y) annotation (Line(
        points={{-50.4,16.4},{10,16.4},{10,2},{24.8,2}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(pulse.y, krev.GlucoseInfusion) annotation (Line(
        points={{-39,-28},{-18,-28},{-18,-5.4},{24.6,-5.4}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(krev.xOutput, slinivka.x) annotation (Line(
        points={{35.8,1.2},{35.8,60},{-28,60},{-28,22.2},{-60.4,22.2}},
        color={0,0,127},
        smooth=Smooth.None));
    annotation (Diagram(graphics));
  end regulaceGlukozy;

  model Krev

  parameter Real QL =   8400 "Glucose Input Flow Rate";
  parameter Real lambda =   2470 "constant";
  parameter Real v = 139000;

    Real TURind "Tissue utilization rate - insulin independent";
    Real TURdep "Tissue utilization rate - insulin dependent";
    Real x "Blood Glucose Concentration (mg/mL)";
    Real RLR;
    parameter Real theta =   2.5 "Reabsorption Maximum";
   parameter Real mi =   7200 "constant";
   parameter Real CG = 15000 "Capacitance of Glucose";
    Modelica.Blocks.Interfaces.RealInput y
      annotation (Placement(transformation(extent={{-72,40},{-32,80}})));
    Modelica.Blocks.Interfaces.RealInput GlucoseInfusion
      annotation (Placement(transformation(extent={{-74,-34},{-34,6}})));
    Modelica.Blocks.Interfaces.RealOutput xOutput
      annotation (Placement(transformation(extent={{48,42},{68,62}})));
  initial equation
    x = 0.81;
    y = 0.055;
  equation
   xOutput=x;
    TURind = lambda * x;
    TURdep = v * x * y;

   if x <= theta then
      RLR = 0;
    else
      RLR = mi * (x - theta);
   end if;

    CG * der(x) = GlucoseInfusion + QL - TURind - TURdep - RLR;

      annotation (Placement(transformation(extent={{-76,40},{-36,80}})), Diagram(
          graphics));
  end Krev;

  model Slinivka

    Modelica.Blocks.Interfaces.RealInput x
      annotation (Placement(transformation(extent={{-84,22},{-44,62}})));
    Modelica.Blocks.Interfaces.RealOutput y
      annotation (Placement(transformation(extent={{26,-26},{46,-6}})));
   parameter Real phi =   0.51 "Threshold Glucose Concentration";
   parameter Real alpha =   7600 "Insulin Destruction Gain";
   parameter Real CI = 15000 "Capacitance of Insulin";
   parameter Real beta =   1430;

  equation
    if x <= phi then
      CI * der(y) = -alpha * y;
    else
      CI * der(y) = -alpha * y + beta * (x - phi);
    end if;

    annotation (Diagram(graphics));
  end Slinivka;

  model Regulation
    glucoseEquation glucoseEquation1
      annotation (Placement(transformation(extent={{8,14},{28,34}})));
    vstup vstup1
      annotation (Placement(transformation(extent={{-76,6},{-56,26}})));
  equation
    connect(vstup1.y, glucoseEquation1.GlucoseInfusion) annotation (Line(
        points={{-62,19.6},{-27,19.6},{-27,27},{8,27}},
        color={0,0,127},
        smooth=Smooth.None));
    annotation (Diagram(graphics));
  end Regulation;

  model vstup

    Modelica.Blocks.Interfaces.RealOutput y
      annotation (Placement(transformation(extent={{30,26},{50,46}})));
  equation
    if (time<0.5) then
                      y=0;
                           else
                               if (time>0.75) then
                                          y=0;
                                   else
                                        y=1000000;
    end if;
            end if;

  end vstup;

  model SimpleGlucose
    glucoseEquation glucoseEquation1(GlucoseInfusion=20)
      annotation (Placement(transformation(extent={{-8,-8},{12,12}})));
    DoseInsertion cukricek(Dose=60, DoseFlow=120)
      annotation (Placement(transformation(extent={{-80,-60},{-60,-40}})));
    DoseInsertion icko(Dose=5000, DoseFlow=50000)
      annotation (Placement(transformation(extent={{-80,20},{-60,40}})));
  equation
    connect(glucoseEquation1.InsulineShot, icko.y) annotation (Line(
        points={{-8,5},{-38,5},{-38,33.6},{-66,33.6}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(cukricek.y, glucoseEquation1.GlucoseShot) annotation (Line(
        points={{-66,-46.4},{-30,-46.4},{-30,-3.1},{-7.9,-3.1}},
        color={0,0,127},
        smooth=Smooth.None));
    annotation (Diagram(graphics));
  end SimpleGlucose;

  model DoseInsertion

    Modelica.Blocks.Interfaces.RealOutput y
      annotation (Placement(transformation(extent={{30,26},{50,46}})));
  parameter Real Dose = 2;
  parameter Real DoseFlow = 1;
  Real curDose( start = Dose, fixed = true);
  discrete Real insert(start = 1.0);
  equation

  if insert >.5 then
    der(curDose) = -DoseFlow;
    y = DoseFlow;
  else
    der(curDose) = 0;
    y = 0;
  end if;

  when curDose <= 0 then
    insert = 0.0;
    reinit(curDose, Dose);
  end when;

  //der(y) = -DoseFlow*y;

    annotation (Diagram(graphics));
  end DoseInsertion;
  annotation (uses(Modelica(version="3.2")));
end Frvs;
