within ;
package Frvs
  model Pharmacokinetics

    ConcKompartment concKompartment(vol=10)
      annotation (Placement(transformation(extent={{-2,-4},{18,16}})));
    absorbtion absorbtion1
      annotation (Placement(transformation(extent={{-68,-4},{-48,16}})));
    clearance clearance1
      annotation (Placement(transformation(extent={{0,-40},{20,-20}})));
    tissues tissues1
      annotation (Placement(transformation(extent={{38,-4},{58,16}})));
    pulsar pulsar1(dpd=6)
      annotation (Placement(transformation(extent={{-100,8},{-80,28}})));
    tissues tissues2
      annotation (Placement(transformation(extent={{40,50},{60,70}})));
  equation
    connect(absorbtion1.massConc1, concKompartment.massConc1) annotation (Line(
        points={{-58,6},{8,6}},
        color={0,0,0},
        pattern=LinePattern.None,
        smooth=Smooth.None));
    connect(concKompartment.massConc1, clearance1.massConc1) annotation (Line(
        points={{8,6},{10,6},{10,-30}},
        color={0,0,0},
        pattern=LinePattern.None,
        smooth=Smooth.None));
    connect(tissues1.massConc1, concKompartment.massConc1) annotation (Line(
        points={{39,6},{8,6}},
        color={0,0,0},
        pattern=LinePattern.None,
        smooth=Smooth.None));
    connect(pulsar1.y, absorbtion1.u) annotation (Line(
        points={{-81,18},{-74,18},{-74,15},{-68,15}},
        color={0,0,127},
        pattern=LinePattern.None,
        smooth=Smooth.None));
    connect(tissues2.massConc1, concKompartment.massConc1) annotation (Line(
        points={{41,60},{8,60},{8,6}},
        color={0,0,255},
        smooth=Smooth.None));
    annotation (Diagram(graphics));
  end Pharmacokinetics;

  model SimplePharmacokinetics

    ConcKompartment concKompartment(vol=10)
      annotation (Placement(transformation(extent={{-2,-4},{18,16}})));
    absorbtion absorbtion1
      annotation (Placement(transformation(extent={{-68,-4},{-48,16}})));
    pulsar pulsar1(dpd=6)
      annotation (Placement(transformation(extent={{-100,8},{-80,28}})));
    clearance clearance1
      annotation (Placement(transformation(extent={{56,-10},{76,10}})));
  equation
    connect(absorbtion1.massConc1, concKompartment.massConc1) annotation (Line(
        points={{-58,6},{8,6}},
        color={0,0,0},
        pattern=LinePattern.None,
        smooth=Smooth.None));
    connect(pulsar1.y, absorbtion1.u) annotation (Line(
        points={{-81,18},{-74,18},{-74,15},{-68,15}},
        color={0,0,127},
        pattern=LinePattern.None,
        smooth=Smooth.None));
    connect(clearance1.massConc1, concKompartment.massConc1) annotation (Line(
        points={{66,0},{38,0},{38,6},{8,6}},
        color={0,0,255},
        smooth=Smooth.None));
    annotation (Diagram(graphics));
  end SimplePharmacokinetics;

  connector massConc
  Real m( unit="mg.l-1");
  flow Real q(unit="mg.h-1");
  end massConc;

  model ConcKompartment

    massConc massConc1
      annotation (Placement(transformation(extent={{-10,-10},{10,10}})));

  parameter Real vol(unit="l") "Distribucni objem";
  Real totalMass( start = 0, unit = "mg");
  constant Real HourSecond( unit="s/h")= 1;
  equation
    totalMass = massConc1.m*vol;
    HourSecond*der(totalMass) = massConc1.q;
    annotation (Diagram(graphics), Icon(graphics={
                                  Rectangle(
            extent={{-80,80},{80,-78}},
            lineColor={255,255,255},
            fillPattern=FillPattern.Sphere,
            fillColor={0,127,0})}));
  end ConcKompartment;

  model flowPump

    massConc massConc1
      annotation (Placement(transformation(extent={{-100,-10},{-80,10}})));
    massConc massConc2
      annotation (Placement(transformation(extent={{80,-10},{100,10}})));

  parameter Real absorptionRate( unit="l/h")= 0.01;
  parameter Real adsorptionRate( unit="l/h")= 10;
  parameter Boolean enabled = true;
  Real concGrad( unit="mg/l");
  equation

  massConc1.q + massConc2.q = 0;
  concGrad = massConc1.m - massConc2.m;

  if not enabled then
    massConc1.q = 0;
  elseif massConc1.q > 0 then
    // in
    massConc1.q/absorptionRate = concGrad;
  else
    //out
      massConc1.q/absorptionRate = concGrad;
  end if;

    annotation (Diagram(graphics), Icon(graphics={
                                  Rectangle(
            extent={{-100,20},{100,-22}},
            lineColor={255,255,255},
            fillPattern=FillPattern.Sphere,
            fillColor={255,0,128}), Polygon(
            points={{-60,20},{40,0},{-60,-22},{-60,-14},{-60,20}},
            lineColor={255,255,255},
            fillPattern=FillPattern.Sphere,
            smooth=Smooth.None,
            fillColor={0,0,0})}));
  end flowPump;

  model absorbtion

    Modelica.Blocks.Interfaces.RealInput u(unit = "mg/h")
      annotation (Placement(transformation(extent={{-120,70},{-80,110}})));
    massConc massConc1
      annotation (Placement(transformation(extent={{-10,-10},{10,10}})));
  Real totalMass( unit="mg");
  parameter Real abs( unit="h") = 1;
  constant Real HourSecond( unit="s/h")= 1;
  equation
    HourSecond*der(totalMass) = u + massConc1.q;
    massConc1.q*abs = -totalMass;
    annotation (Diagram(graphics), Icon(graphics={Polygon(
            points={{-92,80},{-20,-4},{-52,-20},{-10,-40},{28,-34},{34,8},{22,
                46},{0,18},{-84,88},{-84,80},{-92,80}},
            pattern=LinePattern.None,
            fillPattern=FillPattern.Sphere,
            smooth=Smooth.None,
            fillColor={0,0,0})}));
  end absorbtion;

  model clearance

    massConc massConc1
      annotation (Placement(transformation(extent={{-10,-10},{10,10}})));
  parameter Real clearance( unit="ml/min") = 100;
  Real clUnits( unit = "l/h");
  //parameter Real minClearance = 0.01;
  //parameter Real minConc = 0.1;
  equation
  //if massConc1.m > minConc then
    clUnits =  clearance/100*6;
    massConc1.q/clUnits = massConc1.m;
  // else
  //   massConc1.q*minClearance = massConc1.m;
  // end if;

    annotation (Diagram(graphics), Icon(graphics={
          Rectangle(
            extent={{50,-60},{90,-100}},
            pattern=LinePattern.None,
            fillColor={255,255,0},
            fillPattern=FillPattern.Solid,
            lineColor={0,0,0},
            lineThickness=1),
          Line(
            points={{44,-56},{50,-56},{50,-100},{90,-100},{90,-56},{94,-56}},
            smooth=Smooth.None,
            color={0,0,0},
            thickness=1),
          Polygon(
            points={{-26,14},{20,-26},{14,-32},{32,-36},{44,-36},{44,-20},{34,-6},
                {26,-20},{-18,22},{-18,14},{-26,14}},
            pattern=LinePattern.None,
            fillPattern=FillPattern.Sphere,
            smooth=Smooth.None,
            fillColor={0,0,0})}));
  end clearance;

  model tissues
    massConc massConc1 annotation (Placement(transformation(extent={{-100,-10},
              {-80,10}}), iconTransformation(extent={{-100,-10},{-80,10}})));
    ConcKompartment concKompartment(vol=0.01)
      annotation (Placement(transformation(extent={{20,-10},{40,10}})));
    flowPump flowPump1
      annotation (Placement(transformation(extent={{-40,-10},{-20,10}})));
  equation
    connect(flowPump1.massConc1, massConc1) annotation (Line(
        points={{-39,0},{-90,0}},
        color={0,0,0},
        pattern=LinePattern.None,
        smooth=Smooth.None));
    connect(flowPump1.massConc2, concKompartment.massConc1) annotation (Line(
        points={{-21,0},{30,0}},
        color={0,0,0},
        pattern=LinePattern.None,
        smooth=Smooth.None));
    annotation (Diagram(graphics), Icon(graphics={Ellipse(
            extent={{-42,-40},{38,40}},
            pattern=LinePattern.None,
            fillColor={255,0,128},
            fillPattern=FillPattern.Solid,
            lineColor={0,0,0})}));
  end tissues;

  model pulsar
  parameter Real dpd( unit="d-1") = 4 "doses per day";
  discrete Real pulseTime(unit="h", start = startTime);
  discrete Real prePt(start = 0);
  parameter Real startTime = 1;
  parameter Real doseLen( unit="h") = 0.01;
  parameter Real dose(unit = "mg") = 1 "Total dose amount";
  constant Real HourSecond( unit="h/s") = 1;
  Real doseFlow;
  Real doseInterval;
    Modelica.Blocks.Interfaces.RealOutput y annotation (Placement(transformation(
            extent={{80,-10},{100,10}}), iconTransformation(extent={{80,-10},{100,
              10}})));
  equation
  //   when integer((time - startTime)/period)>pre(count) then
  //     count = pre(count)+1;
  //     T_start = time;
  //   end when;
  //   y = offset + (if (time<startTime or nperiod==0 or (nperiod>0 and count>=nperiod)) then 0
  // else if  time<T_start + T_width then amplitude else 0);

    24/dpd = doseInterval;

    when time*HourSecond > pre(pulseTime) + doseLen then
      prePt = pre(pulseTime);
      pulseTime = time*HourSecond + doseInterval;
    end when;

    dose = doseLen*doseFlow;
    y = if pulseTime < time*HourSecond and pulseTime + doseLen > time*HourSecond then doseFlow else 0;

    annotation (Icon(graphics));
  end pulsar;

  model testPulsar
    pulsar pulsar1(dpd=4, dose=100)
      annotation (Placement(transformation(extent={{-42,0},{-22,20}})));
  end testPulsar;
  annotation (uses(Modelica(version="3.2")));
end Frvs;
